/*************************************************************************
 *
 *  $RCSfile: sfx2_map.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: aw $ $Date: 2003/11/18 19:23:17 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
// INCLUDE ---------------------------------------------------------------
#include <stdlib.h>

#pragma hdrstop

#include "map.hxx"
#include "shell.hxx"
#include "objface.hxx"

#include "segmentc.hxx"

#include "dispatch.hxx"
namespace binfilter {

// STATIC DATA -----------------------------------------------------------

DBG_NAME(SfxMap)

SEG_EOFGLOBALS()

extern "C"
#if defined( PM2 ) && !defined( CSET ) && !defined( WTC )
int _stdcall
#else
#ifdef WNT
int _cdecl
#else
int
#endif
#endif
SfxCompareMessages_Impl( const void* pSmaller, const void* pBigger );

// -----------------------------------------------------------------------
#pragma SEG_FUNCDEF(map_01)

SfxMap::SfxMap(const SfxShell &rShell, BOOL bRecursiveSearch ):
	SfxWhichMap( TRUE ),
	pInterface( rShell.GetInterface() ),
	pDispatcher( 0 ),
	bRecursive( bRecursiveSearch )
{
	DBG_CTOR(SfxMap, 0);
	DBG_ASSERT(pInterface != 0, "Interface ist NULL");
}

// -----------------------------------------------------------------------
#pragma SEG_FUNCDEF(map_02)

USHORT SfxMap::GetWhich(USHORT nSlot) const
{
	DBG_CHKTHIS(SfxMap, 0);
	const SfxSlot *pSlot = 0;

	if ( !pInterface && !pDispatcher )
		return nSlot;

	const SfxInterface *pIF = pInterface;
	USHORT nShell = 0;
	if ( pDispatcher )
		 pIF = pDispatcher->GetShell(nShell)->GetInterface();

	while ( pIF )
	{
		for ( ; !pSlot && pIF; pIF = pIF->GetGenoType() )
			pSlot = (const SfxSlot *)bsearch(
								&nSlot, pInterface->pMsgs,
								pInterface->nCount, sizeof(SfxSlot),
								SfxCompareMessages_Impl );
		if ( pSlot )
			return pSlot->GetWhich()? pSlot->GetWhich(): nSlot;
		if ( pDispatcher )
		{
			const SfxShell *pSh = pDispatcher->GetShell(++nShell);
			pIF = pSh ? pSh->GetInterface() : 0;
		}
		else
			pIF = 0;
	}

//  Aus den allgemeinen TabPages kann nach Slot-/Whichwerten
//	gefragt werden, fuer die kein Eintrag in der Map existiert,
//	da sie die Applikation nicht unterstuetzt; deshalb keine
//	Assertion.
//	Der alte Wert wird zurueckgeliefert, damit die Applikation
//	bei 1:1 Mappings keine Eintraege in der Slotmap ertellen muss.
//	(EditEngine benutzt IDs 1:1)
//	DBG_ERROR("Slot nicht gefunden");
	return nSlot;
}

// -----------------------------------------------------------------------
#pragma SEG_FUNCDEF(map_03)

USHORT SfxMap::GetSlot(USHORT nWhich) const
{
	DBG_CHKTHIS(SfxMap, 0);
	DBG_ASSERT(nWhich, "Which-Id ist 0");

	if ( !pInterface && !pDispatcher )
		return nWhich;

	const SfxSlot *pSlot = 0;
	const SfxInterface *pIF = pInterface;
	USHORT nShell = 0;
	if ( pDispatcher )
		 pIF = pDispatcher->GetShell(nShell)->GetInterface();

	while ( pIF )
	{
		for ( ; !pSlot && pIF; pIF = pIF->GetGenoType() )
		{
			const SfxSlot *pTempSlot = pInterface->pMsgs;
			for ( USHORT i = 0; i < pInterface->nCount && !pSlot; ++i )
				if ( (pTempSlot+i)->GetWhich() == nWhich )
					pSlot = pTempSlot;
		}

		if ( pSlot )
			return pSlot->GetId();

		if ( pDispatcher )
		{
			const SfxShell *pSh = pDispatcher->GetShell(++nShell);
			pIF = pSh ? pSh->GetInterface() : 0;
		}
		else
			pIF = 0;
	}

//	Kommenar s.o.
//    DBG_ERROR("Slot nicht gefunden");
	return nWhich;
}

// ------------------------------------------------------------------
#pragma SEG_FUNCDEF(map_04)

SfxWhichMap* SfxMap::Clone() const
{
	return new SfxMap(*this);
}

#pragma SEG_EOFMODULE


}
